<?php

	App::uses('VebraAppModel', 'Vebra.Model');
	App::uses('Vebra', 'Vebra.Model');

	/**
	 * @property Vebra $Vebra
	 */
	class Property extends VebraAppModel {

		public $actsAs = array(
			'Routable.Routable' => array(
				'actual' => ':plugin/:controller/:action/:primaryKey',
				'alias' => ':controller/:action/:primaryKey'
			),
			'MetaData.Meta',
		);
		public $imageSlots = array(
			'main' => array(// main property images
				'slots' => 0
			),
			'floorplan' => array(// floorplan images
				'slots' => 0
			),
			'detail_map' => array(// detail map images
				'slots' => 0
			),
			'street_map' => array(// street map images
				'slots' => 0
			),
			'town_map' => array(// town map images
				'slots' => 0
			),
			'epc' => array(// energy efficiency rating images
				'slots' => 0
			),
			'blurred' => array(
				'slots' => 1
			)
		);
		public $documentSlots = array(
			'brochure' => array(// brochure images
				'slots' => 0
			)
		);
		public $hasMany = array(
			'Paragraph',
		);
		public $belongsTo = 'Branch';

		public function getVebraProperties($branchId) {
			$vebra = new Vebra();
			return $vebra->getProperties($branchId);
		}

		public function getVebraProperty($branchId, $propertyId) {
			$vebra = new Vebra();
			return $vebra->getProperty($branchId, $propertyId);
		}

		public function getVebraUpdatedProperties() {
			$vebra = new Vebra();
			return $vebra->getUpdatedProperties();
		}

		public function getMinPrices() {
			return $this->_getPricesFromTextarea(Configure::read('SiteSetting.min_prices'));
		}

		public function getMaxPrices() {
			return $this->_getPricesFromTextarea(Configure::read('SiteSetting.max_prices'));
		}

		protected function _getPricesFromTextarea($text) {
			$prices = explode("\n", $text);
			array_walk($prices, function(&$value) {
				$value = preg_replace('/(\.\d+|[^\d])/', '', $value); // replace any non-numerical characters, or decimal places, with nothing
			});

			$formattedPrices = array();
			foreach ($prices as $key => $value) {
				$numberFormatter = new CakeNumber();
				$formattedPrices[$value] = $numberFormatter->currency($value, 'GBP', array('places' => 0));
			}
			return $formattedPrices;
		}

		public function getMinBedrooms() {
			return $this->_getBedroomsFromTextarea(Configure::read('SiteSetting.min_bedrooms'));
		}

		public function getMaxBedrooms() {
			return $this->_getBedroomsFromTextarea(Configure::read('SiteSetting.max_bedrooms'));
		}

		protected function _getBedroomsFromTextarea($text) {
			$bedrooms = explode("\n", $text);
			array_walk($bedrooms, function(&$value) {
				$value = preg_replace('/(\.\d+|[^\d])/', '', $value); // replace any non-numerical characters, or decimal places, with nothing
			});

			$formattedBedrooms = array();
			foreach ($bedrooms as $key => $value) {
				$formattedBedrooms[$value] = $value;
			}
			return $formattedBedrooms;
		}

		/**
		 * Pass in a database Image record and property ID, and this will do the rest.
		 * Technically we don't need the propertyId, as it's the model_id, but this makes it a bit clearer what we're doing
		 * @param Image|array $image
		 * @param int $propertyId
		 * @return boolean
		 * @throws Exception
		 */
		public function createBlurredImage($image, $propertyId) {
			// Make and save the new image
			switch ($image['type']) {
				case 'image/jpg' || 'image/jpeg':
					$original_image = imagecreatefromjpeg(WWW_ROOT . $image['filepath']);
					break;

				case 'image/png':
					$original_image = imagecreatefrompng(WWW_ROOT . $image['filepath']);
					break;

				case 'image/gif' || 'image/animated':
					$original_image = imagecreatefromgif(WWW_ROOT . $image['filepath']);
					break;

				default:
					throw new Exception('Image type not supported');
					break;
			}

			// blur the crap out of it. Note, this is VERY slow, so make sure it's done on import of properties
			for ($i = 0; $i < 100; $i++) {
				if (!imagefilter($original_image, IMG_FILTER_GAUSSIAN_BLUR)) {
					break; // don't bother keep going if we fail for some reason
				}
			}
			$blurred_image = $original_image;
			preg_match('/^image\/([a-zA-Z]{3,4})$/', $image['type'], $matches);
			$tmp_file = rtrim(ini_get('upload_tmp_dir'), DS) . DS . str_shuffle('abcdefghijklmnopqrstuvwxyz') . '.' . $matches[1];

			switch ($image['type']) {
				case 'image/jpg' || 'image/jpeg':
					$new_image = imagejpeg($blurred_image, $tmp_file);
					break;

				case 'image/png':
					$new_image = imagepng($blurred_image, $tmp_file);
					break;

				case 'image/gif' || 'image/animated':
					$new_image = imagegif($blurred_image, $tmp_file);
					break;

				default:
					throw new Exception('Image type not supported');
					break;
			}


			$imageData = array(
				'model' => 'Property',
				'model_id' => $propertyId,
				'filename' => array(
					'name' => $image['filename'],
					'type' => $image['type'],
					'size' => $image['size'],
					'tmp_name' => $tmp_file,
					'error' => 0,
				),
				'attachment_type' => 'BlurredImage',
				'type' => $image['type'],
			);

			$imageRecord = new Image();
			if ($imageRecord->create($imageData)) {
				$imageRecord->save();
				return true;
			} else {
				$this->log('Failed to save blurred image');
				return false;
			}
		}

	}
